﻿#!/usr/bin/env python3
# -*- coding:utf-8 -*-
#-------------------------------------------------------------------------------
# Name:        RtklibPosProcessing
# Purpose:      RTKLIBの解析結果（posファイル）を処理して、統計値を出力します
#               静止中の測位結果をまとめるのが目的です。
# Abst.:        スクリプトを実行したフォルダ内にあるすべてのposファイルを対象として処理を実行します。
# Author:      morishita
#
# Created:     07/01/2013
# Copyright:   (c) morishita 2013
# Licence:     new BSD
#-------------------------------------------------------------------------------

import os
import glob
import re
import datetime
import timeKM
import GPS
import mathKM

# posファイル内の処理済みデータにヒットする正規表現
rePosPattern = re.compile(timeKM.reDateGroupedPattern.pattern + r'\s+(?P<lat>\d{1,2}[.]\d+)\s+(?P<lon>\d{1,3}[.]\d+)\s+(?P<height>\d{1,5}[.]\d+)\s+(?P<Q>\d)\s+')

def getStatistics(txt, q_min, q_max):
    """ 受け取った文字列のリストを解析し、指定されたQ値の範囲内にあるデータの統計結果を返す
    Return:
        None: 指定範囲のQ値のデータが存在しない
        tuple: 各統計情報; （開始時刻, 時間幅[s], 緯度の統計情報, 経度の統計情報, 高さの統計情報, 統計に使用されたデータ件数, 抽出率, 抽出された中での各Q値の割合）
    """
    tList = []
    latList = []
    lonList = []
    heightList = []
    qList = []
    matchCount = 0
    totalCount = 0
    qCount = [0, 0, 0, 0, 0, 0]                 # Q = 0, 1, 2, 3, 4, 5 に対応しています
    for line in txt:
        matchTest = rePosPattern.search(line)
        if matchTest != None:
            totalCount += 1
            t      = timeKM.getTime(matchTest.group('time'))
            lat    = float(matchTest.group('lat'))
            lon    = float(matchTest.group('lon'))
            height = float(matchTest.group('height'))
            q      = int(matchTest.group('Q'))
            if q_min <= q <= q_max:
                qCount[q] += 1
                matchCount += 1
                tList.append(t)
                latList.append(lat)
                lonList.append(lon)
                heightList.append(height)
                qList.append(q)
    if matchCount >= 26:                         # 統計学上の最低サンプリング数
        latSta = mathKM.sd3(latList)
        lonSta = mathKM.sd3(lonList)
        heightSta = mathKM.sd3(heightList)
        matchRate = matchCount / totalCount
        qRate = [men / matchCount for men in qCount]
        tstart = tList[0]
        tw = tList[-1] - tList[0]
        return (str(tstart), tw.seconds, lonSta, latSta, heightSta, matchCount, matchRate, qRate)
    else:
        return None

def hoge(txt, t1, t2):
    """ デバッグ用の関数その1
    """
    kk = rePosPattern.findall(txt)
    return kk

def post(fname, t1, t2):
    """ RtklibPosProcessingWithTimeListと共通した処理を関数として吐き出させようかと思ったんだけど、現状で困っていないならどうでもいいやと思って構築を途中で放棄した関数
    """
    # ファイルの存在を確認
    if os.path.exists(fnameOfTimeList) == False:        # ファイルの存在が確認できない場合
        return None
    if os.path.isfile(fnameOfTimeList) != True:         # ファイルでなければ
        return None
    # 引数が時刻オブジェクトであるかどうかを確認する
    if (isinstance(t1, datetime.datetime) and isinstance(t2, datetime.datetime)):
        if(t1 < t2):
            # ファイルの読み込み
            fr = open(fnameOfTimeList,"r")
            lines = fr.readlines()
            fr.close()
            if len(lines) == 0:
                return None
    else:
        return None
    return


def main():
    """ 本スクリプトを直接実行すると呼び出される処理です。
    引数はありません。
    同一フォルダ内にある*.posファイルを処理対象ファイルとして認識します。
    """
    """
    # for debug
    testStr = "2012/02/22 14:00:15.000   32.813676663  130.729680606    88.3109   5  10   1.3315   1.2263   2.4385   0.0980   0.4358   0.7210   0.00    0.0"
    matchTest = rePosPattern.search(testStr)
    if matchTest != None:
        print(matchTest.groups())
        print(rePosPattern.pattern)
        print(matchTest.group('time'))
    testStr2 = testStr + "abababa\n" + testStr
    tako = hoge(testStr2, 1, 1)
    """
    print("Program start...")
    print("This program maintain statistics from RTKLIB log file.\n")

    fw = open("log.csv", "w")
    fw2 = open("log lite.csv", "w")
    filelist = glob.glob(os.getcwd() + '/*.pos')    # 処理対象のファイル一覧を取得
    fw.write("file name,開始時刻,時間幅[s],経度の統計情報,,,,緯度の統計情報,,,,高さの統計情報,,,,統計に使用されたデータ件数,抽出率,抽出された中での各Q値の割合,,,,," + "\n")
    fw.write("file name,start time,time width,mean,sd,use rate,distribution bias, mean,sd,use rate,distribution bias,mean,sd,use rate,distribution bias,統計に使用されたデータ件数,抽出率,Q=0,Q=1,Q=2,Q=3,Q=4,Q=5" + "\n")
    for file in filelist:
        print(os.path.basename(file))
        fr = open(file, "r")
        lines = fr.readlines()
        fr.close()
        #if "10.pos" in file: # for debug
        #    pass
        result = getStatistics(lines, 1, 1)
        print(result)
        if result == None:
            fw2.write(os.path.basename(file) + ",\n")   # 後処理のために、空欄とする
            fw.write(os.path.basename(file) + ",\n")
        else:
            out = str(result)
            out = re.sub("[\'()\[\]]", "", out)
            fw.write(os.path.basename(file) + "," + out + "\n")
            log = ""
            log += "," + str(result[0])
            log += "," + str(result[1])
            log += "," + str(result[2][0][0])           # lon
            log += "," + str(result[3][0][0])           # lat
            log += "," + str(result[4][0][0])           # height
            log += "," + str(result[2][0][1])           # lon sd
            log += "," + str(result[3][0][1])           # lat sd
            log += "," + str(result[4][0][1])           # height sd
            fw2.write(os.path.basename(file) + log + "\n")
    fw.close()
    fw2.close()

if __name__ == '__main__':
    main()
